package com.ruoyi.web.controller.app;

import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.dto.BasicConfigDTO;
import com.ruoyi.common.core.domain.entity.User;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.ss.user.service.IUserService;
import io.swagger.annotations.Api;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import javax.annotation.Resource;

import static com.ruoyi.common.constant.ServiceConstants.USER_TYPE_CLEANER;

/**
 * 用户Controller
 * @author qzz
 * 2024/3/5
 */
@Api(tags = "用户")
@RestController
@RequestMapping("/app/user")
public class AppUserController extends BaseController {

    @Resource
    private IUserService eUserService;

    /**
     * 新增保洁员
     */
    @Log(title = "用户管理", businessType = BusinessType.INSERT)
    @PostMapping("/addCleaner")
    public AjaxResult add(@Validated @RequestBody User user)
    {
        user.setUserName(user.getPhonenumber());
        if (!eUserService.checkUserNameUnique(user))
        {
            return error("新增用户'" + user.getUserName() + "'失败，登录账号已存在");
        }
        else if (StringUtils.isNotEmpty(user.getPhonenumber()) && !eUserService.checkPhoneUnique(user))
        {
            return error("新增用户'" + user.getUserName() + "'失败，手机号码已存在");
        }
        user.setCreateBy(getUsername());
        user.setUserType(USER_TYPE_CLEANER);
        return toAjax(eUserService.insertUser(user));
    }

    /**
     * 保存基础配置
     */
    @Log(title = "保存基础配置", businessType = BusinessType.UPDATE)
    @PostMapping("/basicConfig")
    public AjaxResult basicConfig(@Validated @RequestBody BasicConfigDTO dto)
    {
        logger.info("保存基础配置：【{}】", JSON.toJSONString(dto));
        return toAjax(eUserService.basicConfig(dto));
    }

}
