package com.ruoyi.ss.equipment.service.impl;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.equipment.domain.Equipment;
import com.ruoyi.ss.equipment.domain.EquipmentQuery;
import com.ruoyi.ss.equipment.domain.EquipmentVO;
import com.ruoyi.ss.equipment.mapper.EquipmentMapper;
import com.ruoyi.ss.equipment.service.IEquipmentService;
import com.ruoyi.ss.room.domain.RoomVO;
import com.ruoyi.ss.room.service.IRoomService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;

/**
 * 设施Service业务层处理
 *
 * @author qzz
 * @date 2024-12-21
 */
@Service
public class EquipmentServiceImpl implements IEquipmentService
{
    @Autowired
    private EquipmentMapper equipmentMapper;

    @Autowired
    private IRoomService roomService;
    /**
     * 查询设施
     *
     * @param equipmentId 设施主键
     * @return 设施
     */
    @Override
    public EquipmentVO selectEquipmentByEquipmentId(Long equipmentId)
    {
        return equipmentMapper.selectEquipmentByEquipmentId(equipmentId);
    }

    /**
     * 查询设施列表
     *
     * @param equipment 设施
     * @return 设施
     */
    @Override
    public List<EquipmentVO> selectEquipmentList(EquipmentQuery equipment)
    {
        return equipmentMapper.selectEquipmentList(equipment);
    }


    /**
     * 根据房间id查询设施列表
     *
     * @param roomId 房间id
     * @return 设施集合
     */
    @Override
    public List<EquipmentVO> selectEquipmentListByRoomId(Long roomId) {
        return equipmentMapper.selectEquipmentListByRoomId(roomId);
    }

    /**
     * 新增设施
     *
     * @param equipment 设施
     * @return 结果
     */
    @Override
    public int insertEquipment(Equipment equipment)
    {
        equipment.setCreateTime(DateUtils.getNowDate());
        RoomVO roomVO = roomService.selectERoomByRoomId(equipment.getRoomId());
        ServiceUtil.assertion(roomVO == null, "房间不存在");
        equipment.setStoreId(roomVO.getStoreId());
        equipment.setStoreName(roomVO.getStoreName());
        return equipmentMapper.insertEquipment(equipment);
    }

    /**
     * 修改设施
     *
     * @param equipment 设施
     * @return 结果
     */
    @Override
    public int updateEquipment(Equipment equipment)
    {
        return equipmentMapper.updateEquipment(equipment);
    }

    /**
     * 批量删除设施
     *
     * @param equipmentIds 需要删除的设施主键
     * @return 结果
     */
    @Override
    public int deleteEquipmentByEquipmentIds(Long[] equipmentIds)
    {
        int i = equipmentMapper.deleteEquipmentByEquipmentIds(equipmentIds);
        if (i > 0) {
            equipmentMapper.deleteEquipmentByEquipmentIds(equipmentIds);
        }
        return i;
    }

    /**
     * 删除设施信息
     *
     * @param equipmentId 设施主键
     * @return 结果
     */
    @Override
    public int deleteEquipmentByEquipmentId(Long equipmentId)
    {
        return equipmentMapper.deleteEquipmentByEquipmentId(equipmentId);
    }

    /**
     * 设备投放的房间id
     * @param deviceId 设备id
     * @return String
     */
    @Override
    public List<Long> selectRoomIdByDeviceId(Long deviceId) {
        return equipmentMapper.selectRoomIdByDeviceId(deviceId);
    }

    /**
     * 通过房间id查询所有设备id
     * @param roomId 房间id
     * @return String
     */
    @Override
    public List<Long> selectDeviceIdListByRoomId(Long roomId) {
        return equipmentMapper.selectDeviceIdListByRoomId(roomId);
    }


    @Override
    public List<Long> selectAllEquipmentIdsByRoomId(Long roomId) {
        return equipmentMapper.selectAllEquipmentIdsByRoomId(roomId);
    }

    /**
     * 校验店铺是否已经绑定了大门
     * @param storeId 店铺id
     * @return String
     */
    @Override
    public boolean uniqueGate(Long storeId) {
        return equipmentMapper.uniqueGate(storeId);
    }

    /**
     * 根据店铺查询大门
     * @param storeId 店铺id
     * @return String
     */
    @Override
    public EquipmentVO selectGate(Long storeId) {
        return equipmentMapper.selectGate(storeId);
    }

    /**
     * 根据设备id查询店铺
     * @param deviceId 设备id
     * @return String
     */
    @Override
    public Long selectStoreIdByDeviceId(Long deviceId) {
        return equipmentMapper.selectStoreIdByDeviceId(deviceId);
    }

    /**
     * 根据设备id查询设施对象
     * @param deviceId 设备id
     * @return String
     */
    @Override
    public EquipmentVO selectByDeviceId(Long deviceId) {
        return equipmentMapper.selectByDeviceId(deviceId);
    }

    /**
     * 下架设备
     * @param equipmentId 设施id
     * @return String
     */
    @Override
    public int offlineDevice(Long equipmentId) {
        return equipmentMapper.offlineDevice(equipmentId);
    }


    @Override
    public int selectIdleEquipmentNumByStoreId(Long storeId) {
        return equipmentMapper.selectIdleEquipmentNumByStoreId(storeId);
    }

    /**
     * 根据房间id查询门
     * @param roomId 房间id
     * @param type 设施类型
     * @return String
     */
    @Override
    public EquipmentVO selectDoorByRoomId(Long roomId,String type) {
        return equipmentMapper.selectDoorByRoomId(roomId,type);
    }


}
