package com.ruoyi.ss.refund.domain;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.ruoyi.common.annotation.Excel;
import com.ruoyi.common.core.domain.BaseEntity;
import com.ruoyi.common.core.domain.ValidGroup;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.Min;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.Date;

/**
 * 退款订单对象 et_refund
 *
 * @author 邱贞招
 * @date 2024-05-10
 */
@Data
public class Refund extends BaseEntity
{
    private static final long serialVersionUID = 1L;

    /** 主键 */
    private Long id;

    /** 退款单号 */
    @Excel(name = "退款单号")
    private String refundNo;

    /** 关联订单 */
    @ApiModelProperty("关联订单号")
    @NotBlank(message = "订单号不能为空", groups = {ValidGroup.Create.class})
    private String orderNo;

    /** 用户ID */
    @ApiModelProperty("用户ID")
    private Long userId;

    /** 用户 */
    @ApiModelProperty("用户名")
    private String userName;

    @ApiModelProperty("状态：1-待处理；2-退款成功；3-驳回")
    private String status;

    /** 退款金额 */
    @ApiModelProperty("退款金额")
    @NotNull(message = "退款金额不能为空", groups = {ValidGroup.Create.class})
    @Min(value = 0, message = "退款金额不能小于0元", groups = {ValidGroup.Create.class})
    private BigDecimal amount;

    /** 退款原因 */
    @Excel(name = "退款原因")
    @ApiModelProperty("退款原因")
    private String reason;

    /** 退款结果（回调） */
    @ApiModelProperty("退款结果（回调）")
    private String refundResult;

    /** 商户id */
    @ApiModelProperty("商户id")
    private Long merchantId;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @Excel(name = "处理时间", width = 30, dateFormat = "yyyy-MM-dd HH:mm:ss")
    @ApiModelProperty("处理时间")
    private Date operTime;

}
