package com.ruoyi.ss.refund.service.impl;

import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.ss.order.domain.OrderVO;
import com.ruoyi.ss.order.service.IOrderService;
import com.ruoyi.ss.refund.domain.Refund;
import com.ruoyi.ss.refund.mapper.RefundMapper;
import com.ruoyi.ss.refund.service.IRefundValidator;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Objects;

import static com.ruoyi.common.constant.ServiceConstants.ORDER_PAY_STATUS_NON_PAYMENT;
import static com.ruoyi.common.constant.ServiceConstants.ORDER_STATUS_COMPLETED;

/**
 * 退款订单Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-10
 */
@Service
public class RefundValidatormpl implements IRefundValidator
{
    @Resource
    private RefundMapper refundMapper;

    @Resource
    private IOrderService orderService;

    @Override
    public void preRefund(Refund refund) {
        //
        OrderVO orderVO = orderService.selectRlOrderByOrderNo(refund.getOrderNo());
        ServiceUtil.assertion(orderVO == null, "订单不存在");
        ServiceUtil.assertion(!Objects.equals(orderVO.getUserId(), SecurityUtils.getUserId()), "不是本人订单不能操作");
        ServiceUtil.assertion(ORDER_PAY_STATUS_NON_PAYMENT.equals(orderVO.getPaid()), "订单未支付，不能发起退款");
        ServiceUtil.assertion(refund.getAmount().compareTo(orderVO.getPayFee()) > 0, "退款金额不能超过订单金额");
    }
}
