package com.ruoyi.ss.store.service.impl;

import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.ss.carousel.service.CarouselService;
import com.ruoyi.ss.city.domain.City;
import com.ruoyi.ss.city.service.ICityService;
import com.ruoyi.ss.device.domain.vo.DeviceVO;
import com.ruoyi.ss.device.service.DeviceService;
import com.ruoyi.ss.equipment.domain.Equipment;
import com.ruoyi.ss.equipment.domain.EquipmentVO;
import com.ruoyi.ss.equipment.service.IEquipmentService;
import com.ruoyi.ss.room.domain.Room;
import com.ruoyi.ss.room.domain.RoomVO;
import com.ruoyi.ss.room.service.IRoomAssembler;
import com.ruoyi.ss.room.service.IRoomService;
import com.ruoyi.ss.store.domain.*;
import com.ruoyi.ss.store.mapper.StoreMapper;
import com.ruoyi.ss.store.service.IStoreAssembler;
import com.ruoyi.ss.store.service.IStoreService;
import com.ruoyi.ss.user.service.IUserService;
import com.ruoyi.system.service.ISysConfigService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;

import static com.ruoyi.common.constant.ServiceConstants.*;

/**
 * 店铺Service业务层处理
 *
 * @author qzz
 * @date 2024-11-20
 */
@Slf4j
@Service
public class StoreServiceImpl implements IStoreService
{
    @Autowired
    private StoreMapper storeMapper;

    @Resource
    private ICityService cityService;

    @Resource
    public IRoomService roomService;

    @Autowired
    private DeviceService smDeviceService;

    @Autowired
    private IRoomAssembler roomAssembler;

    @Autowired
    private ISysConfigService configService;

    @Autowired
    private IUserService userService;

    @Autowired
    private IStoreAssembler storeAssembler;

    @Autowired
    private CarouselService carouselService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IEquipmentService equipmentService;

    /**
     * 查询店铺
     *
     * @param storeId 店铺主键
     * @return 店铺
     */
    @Override
    public StoreVO selectEStoreByStoreId(Long storeId)
    {
        StoreVO storeVO = storeMapper.selectEStoreByStoreId(storeId);
        ServiceUtil.assertion(storeVO == null, "店铺不存在");
        List<RoomVO> eRoomVOS = roomService.selectERoomListByStoreId(storeId);
        roomAssembler.assembleRuleInfo(eRoomVOS);
        storeVO.setRoomList(eRoomVOS);
        storeVO.setCarouselList(carouselService.selectSsCarouselListByStoreId(storeVO.getStoreId()));
        return storeVO;
    }

    /**
     * 查询店铺
     *
     * @param storeId 店铺主键
     * @return 店铺
     */
    @Override
    public StoreVO selectMchStoreByStoreId(Long storeId)
    {
        StoreVO storeVO = storeMapper.selectEStoreByStoreId(storeId);
        ServiceUtil.assertion(storeVO == null, "店铺不存在");
        List<RoomVO> eRoomVOS = roomService.selectERoomListByStoreId(storeId);
        roomAssembler.assembleDeviceInfo(eRoomVOS);
        storeVO.setRoomList(eRoomVOS);
        storeVO.setIdleRoomNum(roomService.selectIdleRoomNum(storeId));
        storeVO.setIdleEquipmentNum(equipmentService.selectIdleEquipmentNumByStoreId(storeId));
        storeVO.setToiletNum(roomService.selectToiletNum(storeId));
        return storeVO;
    }

    /**
     * 查询店铺列表
     *
     * @param eStore 店铺
     * @return 店铺
     */
    @Override
    public List<StoreVO> selectEStoreList(StoreQuery eStore)
    {
        return storeMapper.selectEStoreList(eStore);
    }

    /**
     * 新增店铺
     *
     * @param eStore 店铺
     * @return 结果
     */
    @Override
    public int insertEStore(Store eStore)
    {
        eStore.setCreateTime(DateUtils.getNowDate());
        eStore.setMerchantId(SecurityUtils.getUserId());
        if(eStore.getLng() != null && eStore.getLat()!= null){
            eStore.setCityId(cityService.getCityIdByLocation(eStore.getLng().toString(), eStore.getLat().toString()));
        }
        Boolean execute = transactionTemplate.execute(e -> {
            int i = storeMapper.insertEStore(eStore);
            if(i==0)throw new ServiceException("新建店铺失败");
            // 创建大门
            createGate(eStore.getStoreId(), eStore.getName());
            // 创建大厅
            createHall(eStore);
            // 创建卫生间
            createToilet(eStore);
            return Boolean.TRUE;
        });
        if(Boolean.FALSE.equals(execute))throw new ServiceException("新建店铺失败");
        return 1;
    }

    // 创建大厅
    private void createHall(Store eStore) {
        // 默认新增一个大厅
        Room room = new Room();
        room.setRoomName("大厅");
        room.setType(eStore.getType());
        room.setType2(ROOM_TYPE2_HALL);
        room.setStoreId(eStore.getStoreId());
        room.setStoreName(eStore.getName());
        room.setStatus(ROOM_STATUS_FREE);
        int i = roomService.insertERoom(room);
        ServiceUtil.assertion(i==0,"新增大厅失败");
    }

    // 创建卫生间
    private void createToilet(Store eStore) {
        // 默认新增一个卫生间
        Room room = new Room();
        room.setRoomName("卫生间");
        room.setType(eStore.getType());
        room.setType2(ROOM_TYPE2_BATHROOM);
        room.setStoreId(eStore.getStoreId());
        room.setStoreName(eStore.getName());
        room.setStatus(ROOM_STATUS_FREE);
        int i = roomService.insertERoom(room);
        ServiceUtil.assertion(i==0,"新增卫生间失败");
    }

    /**
     * 修改店铺
     *
     * @param eStore 店铺
     * @return 结果
     */
    @Override
    public int updateEStore(Store eStore)
    {
        eStore.setUpdateTime(DateUtils.getNowDate());
        return storeMapper.updateEStore(eStore);
    }

    /**
     * 批量删除店铺
     *
     * @param storeIds 需要删除的店铺主键
     * @return 结果
     */
    @Override
    public int deleteEStoreByStoreIds(Long[] storeIds)
    {
        return storeMapper.deleteEStoreByStoreIds(storeIds);
    }

    /**
     * 删除店铺信息
     *
     * @param storeId 店铺主键
     * @return 结果
     */
    @Override
    public int deleteEStoreByStoreId(Long storeId)
    {
        // todo 店铺下是否有设施，不能删除
        return storeMapper.deleteEStoreByStoreId(storeId);
    }

    /**
     * 根据定位获取附近店铺列表
     */
    @Override
    public List<StoreVO> getStoreListByLocation(StoreQuery query) {
        // 根据定位查询出是哪个城市，获取该城市下的所有店铺
        City city = cityService.getCityByLocation(query.getPhoneLon(), query.getPhoneLat());
        if(!Objects.isNull(city)){
            List<StoreVO> eStoreVOS = storeMapper.selectEStoreByCityId(city.getCityId());
            for (StoreVO eStoreVO : eStoreVOS) {
                eStoreVO.setDistance(getDistance(query.getPhoneLon(), query.getPhoneLat(), eStoreVO.getLng(), eStoreVO.getLat()));
            }
            return eStoreVOS;
        }
        return null;
    }

    /**
     * 根据城市id获取店铺列表
     */
    @Override
    public List<StoreVO> getStoreListByCityId(Long cityId) {
        return storeMapper.selectEStoreByCityId(cityId);
    }


    /**
     * 开大门
     */
    @Override
    public boolean openGate(Long storeId) {
        EquipmentVO equipmentVO = equipmentService.selectGate(storeId);
        ServiceUtil.assertion(equipmentVO == null, "大门设施不存在");
        return smDeviceService.switchDevice(equipmentVO.getDeviceId(), true);
    }

    /**
     * 关大门
     */
    @Override
    public boolean closeGate(Long storeId) {
        EquipmentVO equipmentVO = equipmentService.selectGate(storeId);
        ServiceUtil.assertion(equipmentVO == null, "大门设施不存在");
        return smDeviceService.switchDevice(equipmentVO.getDeviceId(), false);
    }

    /**
     * 客服电话
     */
    @Override
    public ServerPhone getServicePhone(Long storeId) {
        StoreVO eStoreVO = storeMapper.selectEStoreByStoreId(storeId);
        ServerPhone serverPhone = new ServerPhone();
        serverPhone.setStorePhone(eStoreVO.getServerPhone());
        String platformPhone = configService.selectConfigByKey("ss.platform.phone");
        serverPhone.setPlatformPhone(platformPhone);
        return serverPhone;
    }

    /**
     * 查看可用门店
     */
    @Override
    public List<StoreVO> getAvailableStores(Long storeId) {
        StoreVO eStoreVO = storeMapper.selectEStoreByStoreId(storeId);
        return storeMapper.getStoreListByMerchantId(eStoreVO.getMerchantId());
    }

    /**
     * 绑定大门
     */
    @Override
    public int bandGate(StoreDTO dto) {
        // 根据sn查询设备
        DeviceVO deviceVO = smDeviceService.selectById(dto.getDeviceId());
        ServiceUtil.assertion(deviceVO == null, "设备不存在");
        StoreVO storeVO = storeMapper.selectEStoreByStoreId(dto.getStoreId());
        ServiceUtil.assertion(storeVO == null, "店铺不存在");
        ServiceUtil.assertion(equipmentService.uniqueGate(dto.getStoreId()), "该店铺已经绑定了大门");

        EquipmentVO equipmentVO = equipmentService.selectGate(dto.getStoreId());
        ServiceUtil.assertion(equipmentVO == null, "该店铺未查询到大门设施");
        Equipment equipment = new Equipment();
        equipment.setDeviceId(dto.getDeviceId());
        equipment.setEquipmentId(equipmentVO.getEquipmentId());
        int i = equipmentService.updateEquipment(equipment);
        ServiceUtil.assertion(i == 0, "绑定大门失败");
        return i;
    }

    private int createGate(Long storeId, String storeName) {
        Equipment equipment = new Equipment();
        equipment.setName("大门");
        equipment.setStoreId(storeId);
        equipment.setStoreName(storeName);
        equipment.setType(FACILITY_TYPE_GATE);
        int i = equipmentService.insertEquipment(equipment);
        ServiceUtil.assertion(i==0,"绑定大门失败");
        return i;
    }

    /**
     * 更新门店类型标签
     */
    @Override
    public int updateStoreTypeTags(Long storeId, String type) {
        //该店铺下是否存在该房间标签，不存在则添加后更新，存在则跳过
        boolean isTypeTags = storeMapper.isExistTypeTags(storeId,type);
        if(!isTypeTags){
            return storeMapper.updateStoreTypeTags(storeId,type);
        }
        return 1;
    }

    /**
     * 门禁授权
     */
    @Override
    public int gateAuth(GateAuthDTO dto) {
        EquipmentVO gate = equipmentService.selectGate(dto.getStoreId());
        ServiceUtil.assertion(gate == null, "该店铺未查询到大门设施");

        EquipmentVO equipmentVO = new EquipmentVO();
        equipmentVO.setUnlockCondition(dto.getUnlockCondition());
        equipmentVO.setUnlockMode(dto.getUnlockMode());
        equipmentVO.setUnlockTime(dto.getUnlockTime());
        equipmentVO.setEquipmentId(gate.getEquipmentId());
        return equipmentService.updateEquipment(equipmentVO);
    }

    private Double getDistance(String phoneLon, String phoneLat, BigDecimal lng, BigDecimal lat) {
        double[] point1 = {Double.parseDouble(phoneLon), Double.parseDouble(phoneLat)};
        double[] point2 = {lng.doubleValue(), lat.doubleValue()};
        double distanceInMeters = GeoUtils.haversineDistance(point1, point2);
        return Math.round((distanceInMeters / 1000) * 10) / 10.0;
    }
}
