package com.ruoyi.system.task;

import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.YPMsgUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.ss.clean.domain.Clean;
import com.ruoyi.ss.clean.service.ICleanService;
import com.ruoyi.ss.order.domain.Order;
import com.ruoyi.ss.order.domain.OrderVO;
import com.ruoyi.ss.orderOper.service.IOrderOperService;
import com.ruoyi.ss.order.service.IOrderService;
import com.ruoyi.ss.room.domain.Room;
import com.ruoyi.ss.room.mapper.RoomMapper;
import com.ruoyi.ss.room.service.IRoomService;
import com.ruoyi.ss.store.service.IStoreService;
import com.ruoyi.ss.user.domain.UserVO;
import com.ruoyi.ss.user.service.IUserService;
import com.ruoyi.system.service.ISysConfigService;
import com.ruoyi.system.service.IWxPayService;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

import static com.ruoyi.common.constant.ServiceConstants.IS_REFUND_STATUS_PENDING;
import static com.ruoyi.common.constant.ServiceConstants.IS_REFUND_STATUS_REFUNDED;

/**
 * 定时任务调度测试
 *
 * @author ruoyi
 */
@Slf4j
@Component("ssTask")
public class SsTask {


    @Resource
    private IOrderService orderService;

    @Autowired
    private ICleanService cleanService;

    @Autowired
    private IUserService userService;

    @Autowired
    private IRoomService roomService;

    @Resource
    private IWxPayService wxPayService;



    /**
     * 1.启动时判断是否有未取消预约的订单
     * 2.判断已完成的订单未退还押金的
     * 3.启动时判断是否分账
     */
    @Transactional
    @PostConstruct
    public void init() {
    }

    /**
     *  结束订单处理
     *  查询未结束已到期的订单，通知保洁，修改订单状态
     */
    public void orderEndHandler(){
        log.info("-------------------【定时任务】结束订单处理---开始----------------");
        List<OrderVO> orderVOS = orderService.selectEndOrderList();
        for (OrderVO orderVO: orderVOS) {
            if(!orderVO.getIsSendMsg()){//没有通知过去通知保洁。修改订单状态：已结束、通知状态：已通知
                orderVO.setIsSendMsg(true);
                // todo 通知保洁, 发送短信  店铺id,保洁
                String merchantPhone = orderVO.getMerchantPhone();
                YPMsgUtils.sendClearMsg(orderVO.getRoomName(),merchantPhone);

                Clean clean = buildClean(orderVO);
                int i = cleanService.insertEClean(clean);
                ServiceUtil.assertion(i==0,"生成保洁订单失败");
            }
            updateOrderStatus(orderVO);
            // todo 改变房间状态，改为未打扫
            updateRoomStatus(orderVO);
        }
        log.info("-------------------【定时任务】结束订单处理---结束----------------");
    }

    /**
     *  押金自动退还处理 30秒执行一次
     *  查询所有智能押金模式下，还未退款的订单，根据实际退款时间退款
     */
    public void depositAutoRefund(){
        log.info("-------------------【定时任务】押金自动退还处理---开始----------------");
        List<OrderVO> orderVOS = orderService.selectNotRefundedOrderList();
        for (OrderVO orderVO: orderVOS) {
            // todo 是否加锁？判空
            // todo 更新订单是否退款状态
            Order order = new Order();
            order.setIsRefund(IS_REFUND_STATUS_REFUNDED);
            order.setOrderNo(orderVO.getOrderNo());
            int i = orderService.updateRlOrderByOrderNo(orderVO);
            ServiceUtil.assertion(i==0,"更新订单是否退款状态失败");

            String refNo = IdUtils.getOrderNo("ref");
            BigDecimal depositRefund = orderVO.getDepositRefund();
            wxPayService.refund(orderVO,"订单结束，已使用金额："+orderVO.getPayFee()+",退还剩余金额："+depositRefund, depositRefund,refNo);
        }
        log.info("-------------------【定时任务】押金自动退还处理---结束----------------");
    }



    private void updateRoomStatus(OrderVO orderVO) {
        Room room = new Room();
        room.setRoomId(orderVO.getRoomId());
        room.setStatus(ServiceConstants.ROOM_STATUS_UNCLEANED);
        int i = roomService.updateERoom(room);
        ServiceUtil.assertion(i==0,"更新房间状态失败");
    }

    private @NotNull Clean buildClean(OrderVO orderVO) {
        // 根据店铺获取保洁员
        // 生成保洁订单 cleanService
        Clean clean = new Clean();
        List<UserVO> clearUserByStoreId = userService.getClearUserByStoreId(orderVO.getStoreId());
        if(!clearUserByStoreId.isEmpty()){
            UserVO userVO = clearUserByStoreId.get(0);
            clean.setUserId(userVO.getUserId());
            clean.setUserName(userVO.getUserName());
            clean.setUserPhone(userVO.getPhonenumber());
        }
        clean.setStartTime(DateUtils.getNowDate());
        clean.setMerchantId(orderVO.getMerchantId());
        clean.setRoomId(orderVO.getRoomId());
        clean.setRoomName(orderVO.getRoomName());
        clean.setStatus(ServiceConstants.STATUS_PENDING_CLEANING);
        return clean;
    }

    private void updateOrderStatus(OrderVO orderVO) {
        orderVO.setStatus(ServiceConstants.ORDER_STATUS_COMPLETED);
        int i = orderService.updateRlOrderByOrderNo(orderVO);
        ServiceUtil.assertion(i==0,"更新订单状态失败");
    }

}
